@@ -15,13 +15,6 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with gracedb.  If not, see <http://www.gnu.org/licenses/>.
+from .version import __version__
 
-__all__ = ["cli", "rest"]
-
-GIT_TAG = 'gracedb-1.28-1'
-
-# issue 717.  Required for backward compatibility -- make sure "from ligo import gracedb"
-# works as it used to.
-from .cli import *
-from .rest import ProxyHTTPConnection, ProxyHTTPSConnection
-
+__all__ = ["cli", "exceptions", "rest", "utils"]
