@@ -19,14 +19,9 @@
 from __future__ import print_function
 import os, sys, shutil
 import json
-from ligo.gracedb.rest import GraceDb, GraceDbBasic
 import six
-
-DEFAULT_SERVICE_URL = "https://gracedb.ligo.org/api"
-
-DEFAULT_BASIC_URL = "https://gracedb.ligo.org/apibasic/"
-
-GIT_TAG = 'gracedb-1.28-1'
+from ligo.gracedb.rest import GraceDb
+from ligo.gracedb.rest import DEFAULT_SERVICE_URL
 
 DEFAULT_COLUMNS = "graceid,labels,group,pipeline,search,far,gpstime,created,dataurl"
 
@@ -47,13 +42,13 @@ def output(*message):
 
 def defaultAccess(e,a):
     if a.find('.') < 0:
-        return str(e.get(a,""))
+        return repr(e.get(a,""))
     rv = e
     attrs = a.split('.')
     while attrs and rv:
         rv = rv.get(attrs[0],"")
         attrs = attrs[1:]
-    return str(rv)
+    return repr(rv)
 
 #-----------------------------------------------------------------
 # HTTP upload encoding
@@ -78,61 +73,55 @@ typeCodeMap = {
 validTypes = list(typeCodeMap.keys())
 
 #-----------------------------------------------------------------
-# This is a factory for client classes.
-# Given a base client class with the correct properties, derive
-# one suitable for use with this command-line tool. In practice,
-# the base class here will either be the X509 auth GraceDb class
-# or the basic auth GraceDbBasic class.
+# Override and add a few methods to the base GraceDb client class
+class GraceDbClient(GraceDb):
+    def __init__(self, url=DEFAULT_SERVICE_URL, *args, **kwargs):
+        if (url[-1] != '/'):
+            url += '/'
+        self.url = url
+        super(GraceDbClient, self).__init__(url, *args, **kwargs)
 
-def derive_client(ClientBase=GraceDb):
-    class client(ClientBase):
-        def __init__(self, url=DEFAULT_SERVICE_URL, *args, **kwargs):
-            if (url[-1] != '/'):
-                url += '/'
-            self.url = url
-            super(client, self).__init__(url, *args, **kwargs)
+    def download(self, graceid, filename, destfile):
+        # Check that we *could* write the file before we
+        # go to the trouble of getting it.  Also, try not
+        # to open a file until we know we have data.
+        if not hasattr(destfile, 'read') and destfile != "-":
+            if not os.access(os.path.dirname(os.path.abspath(destfile)), os.W_OK):
+                raise IOError("%s: Permission denied" % destfile)
+        response = self.files(graceid, filename)
+        if response.status == 200:
+            if not hasattr(destfile, 'read'):
+                if destfile == '-':
+                    destfile = sys.stdout
+                    # Python 2/3 compatibility
+                    if hasattr(destfile, 'buffer'):
+                        destfile = destfile.buffer
+                else:
+                    destfile = open(destfile, "wb")
+            shutil.copyfileobj(response, destfile)
+            return 0
+        else:
+            return "Error. (%d) %s" % (response.status, response.reason)
 
-        def download(self, graceid, filename, destfile):
-            # Check that we *could* write the file before we
-            # go to the trouble of getting it.  Also, try not
-            # to open a file until we know we have data.
-            if not hasattr(destfile, 'read') and destfile != "-":
-                if not os.access(os.path.dirname(os.path.abspath(destfile)), os.W_OK):
-                    raise IOError("%s: Permission denied" % destfile)
-            response = self.files(graceid, filename)
-            if response.status == 200:
-                if not hasattr(destfile, 'read'):
-                    if destfile == '-':
-                        destfile = sys.stdout
-                    else:
-                        destfile = open(destfile, "wb")
-                shutil.copyfileobj(response, destfile)
-                return 0
-            else:
-                return "Error. (%d) %s" % (response.status, response.reason)
+    # Hamstring 'adjustResponse' from the example REST client.
+    # We don't want it messing with the response from the server.
+    def adjustResponse(self, response):
+        response.json = lambda: self.load_json_or_die(response)
+        return response
 
-        # Hamstring 'adjustResponse' from the example REST client.
-        # We don't want it messing with the response from the server.
-        def adjustResponse(self, response):
-            response.json = lambda: self.load_json_or_die(response)
-            return response
-
-        @classmethod
-        def output_and_die(cls, msg):
-            sys.stderr.write(msg)
-            sys.exit(1)
-
-    return client
-
-# Get X509 client in global scope
-# This is required for passing the unit tests.
-Client = derive_client()
+    @classmethod
+    def output_and_die(cls, msg):
+        sys.stderr.write(msg)
+        sys.exit(1)
 
 #-----------------------------------------------------------------
 # Main
 
 
-def main():
+def main(args=None):
+    if args is None:
+        args = sys.argv[1:]
+
     usage ="""%%prog [options] GROUP PIPELINE SEARCH EVENTFILE
     where GROUP is one of %(groups)s
          PIPELINE is one of %(pipelines)s
@@ -209,11 +198,12 @@ Credentials are looked for in this order:
     (2) $(X509_USER_PROXY)
     (3) Default location of grid proxy ( /tmp/x509up_u$(UID) )
     (4) $(HOME)/.globus/usercert.pem / $(HOME)/.globus/userkey.pem
+    (5) Basic auth credentials in $(HOME)/.netrc
 
 Note that comments can only be 200 characters long.
 Longer strings will be truncated.""" % {
         'groups'     : 'CBC, Burst, Stochastic, Coherent, Test, External',
-        'pipelines'  : 'MBTAOnline, gstlal, gstlal-spiir, HardwareInjection, Fermi, Swift, CWB, CWB2G',
+        'pipelines'  : 'MBTAOnline, gstlal, spiir, oLIB, PyCBC, HardwareInjection, Fermi, Swift, CWB, CWB2G',
         'searches'   : 'AllSky, LowMass, HighMass, GRB, Test',
     }
 
@@ -225,12 +215,6 @@ Longer strings will be truncated.""" % {
                   help="GraCEDb Service URL", metavar="URL")
     op.add_option("-f", "--filename", dest="filename",
                   help="If data is read from stdin, use this as the filename.", metavar="NAME")
-
-    op.add_option("-a", "--alert", dest="alert",
-                  help="Send an LV alert (deprecated; alerts sent by default)",
-                  action="store_true", default=None
-                 )
-
     op.add_option("-c", "--columns", dest="columns",
                   help="Comma separated list of event attributes to include in results (only meaningful in search)",
                   default=DEFAULT_COLUMNS
@@ -248,20 +232,30 @@ Longer strings will be truncated.""" % {
                   help="tag display name (ignored for existing tags)",
                   default=None
                  )
-    op.add_option("-b", "--use-basic-auth", dest="use_basic_auth",
-                  help="Use basic auth with a .netrc file. Available to non-LVC members only.",
+    op.add_option("-n", "--force-noauth", dest="force_noauth",
+                  help="Do not use any authentication credentials.",
                   action="store_true", default=False
                  )
     op.add_option("-o", "--offline", dest="offline", action="store_true",
                   help=("Signifies that an event was found by an offline "
                   "pipeline. Used when creating a new event."), default=False
                  )
+    op.add_option("-a", "--username", dest="username", type=str,
+                  help=("Username for basic auth"), default=None
+                 )
+    op.add_option("-b", "--password", dest="password", type=str,
+                  help=("Password for basic auth"), default=None
+                 )
     op.add_option("--labels", dest="labels",help=("Defines labels for creating"
                   " a new event. Should be a comma-separated list of labels."),
                   default=[], type=str
                  )
+    op.add_option("--show-creds", dest="show_creds", help=("Prints your "
+                  "authentication credentials and exits."), default=False,
+                  action="store_true"
+                 )
 
-    options, args = op.parse_args()
+    options, args = op.parse_args(args)
     if isinstance(options.labels, str):
         options.labels = options.labels.split(',')
 
@@ -284,26 +278,10 @@ Longer strings will be truncated.""" % {
               os.environ.get('GRACEDB_SERVICE_URL', None) or \
               DEFAULT_SERVICE_URL
 
-    # If the user requested a specific service, but also wants basic auth,
-    # then the service had better be a basic auth endpoint. Otherwise die.
-    # On the other hand, if the user did not specify a service url, then we
-    # will use the default basic URL if basic auth was requested.
-    if options.use_basic_auth:
-        if options.service or os.environ.get('GRACEDB_SERVICE_URL', None):
-            if 'basic' not in service:
-                error("To use the basic auth client, specify a basic auth service URL or use the default.")
-                exit(1)
-        else:
-            service = DEFAULT_BASIC_URL
-
-    # Client subclass according to preferred auth method.
-    global Client
-    if options.use_basic_auth:
-        Client = derive_client(GraceDbBasic)
-
-    if options.alert is not None:
-        warning("alert option is deprecated.  Alerts are now sent by default.")
+    # Compile dict of kwargs for instantiating the client class
+    client_args = {}
 
+    # Process proxy args
     proxyport = None
     if proxy and proxy.find(':') > 0:
         try:
@@ -311,12 +289,21 @@ Longer strings will be truncated.""" % {
             proxyport = int(proxyport)
         except:
             op.error("Malformed proxy: '%s'" % proxy)
-    if proxyport:
-        client = Client(service,
-                        proxy_host=proxy,
-                        proxy_port=proxyport)
-    else:
-        client = Client(service, proxy_host=proxy)
+    client_args['proxy_host'] = proxy
+    client_args['proxy_port'] = proxyport
+
+    # Auth args
+    client_args['force_noauth'] = options.force_noauth
+    client_args['username'] = options.username
+    client_args['password'] = options.password
+
+    # Instantiate the client
+    client = GraceDbClient(service, **client_args)
+
+    if options.show_creds:
+        client.show_credentials()
+        print()
+        exit(0)
 
     if len(args) < 1:
         op.error("not enough arguments")
@@ -401,7 +388,7 @@ Longer strings will be truncated.""" % {
                 tagDispName = args[4]
         graceid = args[1]
         logN = args[2]
-        response = client.createTag(graceid, logN, tagName, tagDispName)
+        response = client.addTag(graceid, logN, tagName, tagDispName)
     elif args[0] == 'delete_tag':
         error("len args = %s" % len(args))
         error("args = %s" % args)
@@ -415,7 +402,7 @@ Longer strings will be truncated.""" % {
             tagName = args[3]
         graceid = args[1]
         logN = args[2]
-        response = client.deleteTag(graceid, logN, tagName)
+        response = client.removeTag(graceid, logN, tagName)
     elif args[0] == 'label':
         if len(args) != 3:
             op.error("wrong number of arguments for label")
@@ -451,7 +438,7 @@ Longer strings will be truncated.""" % {
         else:
             accessFun = {
                 "labels" : lambda e: \
-                    ",".join(list(e['labels'].keys())),
+                    ",".join(list(e['labels'])),
                 "dataurl" : lambda e: e['links']['files'],
             }
 
@@ -515,7 +502,7 @@ Longer strings will be truncated.""" % {
         except:
             rv = response
         try:
-            rv = json.loads(rv)
+            rv = json.loads(rv.decode())
         except:
             pass
 
@@ -540,9 +527,9 @@ Longer strings will be truncated.""" % {
         rv = response
 
     try:
-        responseBody = json.loads(rv)
+        responseBody = json.loads(rv.decode())
     except:
-        responseBody = rv
+        responseBody = rv.decode()
 
     if status >= 400:
         exitCode=1
